#ifndef OSD_TYPE_TELETEXT_H_
#define OSD_TYPE_TELETEXT_H_

#include <vector>
#include <map>
using namespace std;

#include <qstring.h>
#include <qrect.h>
#include <qmap.h>
#include <qcolor.h>
#include <qmutex.h>
#include <qfile.h>
#include <qstring.h>
#include <qtextstream.h>

#include "../libs/libmythtv/teletextdecoder.h"

class SRTOutputFile
{
    public:
                    SRTOutputFile(QString outfile);
                    ~SRTOutputFile(void);
        bool        Open();
        void        Close();
        void        Set_DTS(int64_t dts) { m_dts = dts; }
        void        Set_Enable(bool enabled);
        bool        Is_Enabled() const { return m_enabled; };
        void        Set_Line(QString new_line);
    private:
        QString     Pts_To_Srt_Time(int64_t pts);
        void        Output_Line();
        
        int64_t     m_dts;
        bool        m_opened;
        bool        m_enabled;
        QFile       m_outfp;
        QTextStream m_outstream;
        QString     m_line;
        int64_t     m_line_dts;
        int         m_index;
};


class TTColor
{
  public:
    static const uint BLACK       = 0;
    static const uint RED         = 1;
    static const uint GREEN       = 2;
    static const uint YELLOW      = 3;
    static const uint BLUE        = 4;
    static const uint MAGENTA     = 5;
    static const uint CYAN        = 6;
    static const uint WHITE       = 7;
    static const uint TRANSPARENT = 8;
};

class TTKey
{
  public:
    static const uint k0            = 0;
    static const uint k1            = 1;
    static const uint k2            = 2;
    static const uint k3            = 3;
    static const uint k4            = 4;
    static const uint k5            = 5;
    static const uint k6            = 6;
    static const uint k7            = 7;
    static const uint k8            = 8;
    static const uint k9            = 9;
    static const uint kNextPage     = 10;
    static const uint kPrevPage     = 11;
    static const uint kNextSubPage  = 12;
    static const uint kPrevSubPage  = 13;
    static const uint kHold         = 14;
    static const uint kTransparent  = 15;
    static const uint kFlofRed      = 16;
    static const uint kFlofGreen    = 17;
    static const uint kFlofYellow   = 18;
    static const uint kFlofBlue     = 19;
    static const uint kFlofWhite    = 20;
    static const uint kRevealHidden = 21;
};

#define TP_SUPPRESS_HEADER  0x01
#define TP_UPDATE_INDICATOR 0x02
#define TP_INTERRUPTED_SEQ  0x04
#define TP_INHIBIT_DISPLAY  0x08
#define TP_MAGAZINE_SERIAL  0x10
#define TP_ERASE_PAGE       0x20
#define TP_NEWSFLASH        0x40
#define TP_SUBTITLE         0x80

class TeletextSubPage
{
  public:
    int pagenum;              ///< the wanted page
    int subpagenum;           ///< the wanted subpage
    int lang;                 ///< language code
    int flags;                ///< misc flags
    uint8_t data[25][40];     ///< page data
    int flof;                 ///< page has FastText links
    int floflink[6];          ///< FastText links (FLOF)
    bool subtitle;            ///< page is subtitle page
    bool active;              ///< data has arrived since page last cleared
};
typedef map<int, TeletextSubPage> int_to_subpage_t;

class TeletextPage
{
  public:
    int               pagenum;
    int               current_subpage;
    int_to_subpage_t  subpages;
};
typedef map<int, TeletextPage> int_to_page_t;

#define MAGAZINE(page) (page / 256)

class TeletextMagazine
{
  public:
    int               current_page;
    int               current_subpage;
    TeletextSubPage   loadingpage;
    int_to_page_t     pages;
};

class SRTTypeTeletext : public TeletextViewer
{
  public:
    SRTTypeTeletext();
    virtual ~SRTTypeTeletext() {}

    // TeletextViewer interface methods
    void SetPage(int page, int subpage);

    void Reset(void);
    void AddPageHeader(int page, int subpage,
                       const uint8_t *buf, int vbimode, int lang, int flags);
    void AddTeletextData(int magazine, int row,
                         const uint8_t* buf, int vbimode);
    void SetDisplaying(bool displaying) { m_displaying = displaying; }
    void Set_Output_Device(SRTOutputFile *output_device) { m_output_device = output_device; };

  private:
    // Internal Teletext Commands
    void NewsFlash(void) {};
    void PageUpdated(int page, int subpage);
    void HeaderUpdated(uint8_t *page, int lang);
    bool DecodeLine(const unsigned char *page, uint row, int lang, QString &line) const;

    const TeletextSubPage *FindSubPage(int page, int subpage, int dir=0) const
        { return FindSubPageInternal(page, subpage, dir); }
    TeletextSubPage       *FindSubPage(int page, int subpage, int dir = 0)
        { return (TeletextSubPage*) FindSubPageInternal(page, subpage, dir); }

    const TeletextPage    *FindPage(int page, int dir = 0) const
        { return (TeletextPage*) FindPageInternal(page, dir); }
    TeletextPage          *FindPage(int page, int dir = 0)
        { return (TeletextPage*) FindPageInternal(page, dir); }

    const TeletextSubPage *FindSubPageInternal(int,int,int) const;
    const TeletextPage    *FindPageInternal(int,int) const;

  private:
    int          m_tt_colspace;
    int          m_tt_rowspace;

    // last fetched page
    int          m_fetchpage;
    int          m_fetchsubpage;

    // currently displayed page:
    mutable int  m_curpage;
    mutable int  m_cursubpage;
    mutable bool m_curpage_showheader;
    mutable bool m_curpage_issubtitle;

    int          m_pageinput[3];

    bool         m_transparent;
    bool         m_revealHidden;

    bool         m_displaying;
    bool         m_changed;

    uint8_t      m_header[40];
    mutable bool m_header_changed;
    mutable bool m_page_changed;
    mutable bool m_osd_changed;
    SRTOutputFile *m_output_device;
    
    TeletextMagazine m_magazines[8];
    unsigned char    m_bitswap[256];

    static const int    kTeletextColumns;
    static const int    kTeletextRows;
};



#endif

